#include "Movable.h"
#include "Status.h"
#include "Vram.h"
#include "Stage.h"
#include "Main.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        *pVram = c; ++pVram;
        ++p;
    }
    return pVram;
}

static ptr<byte> PrintRemain(ptr<byte> pVram)
{
    static const byte[] RemainChars = {
        0xc8, 0xfb, 0xfb, 0x84,
    };
    ptr<byte> pChar;
    pChar = RemainChars;
    repeat (4) {
        *pVram = *pChar;
        ++pVram;
        ++pChar;
    }
    return pVram;
}

void PrintStatus() 
{
    PrintS(Vram + 2, "SCORE");
    PrintS(Vram + 16, "HI-SCORE");
    PrintS(Vram + 33, "STAGE");
    PrintByteNumber2(Vram + 39, CurrentStage + 1);
    PrintS(Vram + 43, "TIME");

    {
        ptr<byte> pVram;
        pVram = Vram + 54;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            if (i > 2) {
                pVram = PrintRemain(pVram);
                *pVram = i + '0';
            }
            else {
                do {
                    pVram = PrintRemain(pVram);
                    --i;
                } while (i > 0);
            }
        }
    }
    PrintScore();
    PrintTime();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + 8, Score);
    *pVram = '0';

    pVram = PrintNumber5(Vram + 25, HiScore);
    *pVram = '0';
}

void PrintTime()
{
    PrintByteNumber3(Vram + 48, StageTime);
}


void PrintGameOver()
{
    PrintS(Vram + VramRowSize * 8 + 27, "GAME OVER");
}

void PrintTimeUp() 
{
    PrintS(Vram + VramRowSize * 8 + 28, "TIME UP");
}

void Title()
{
    static const byte[] TitleBytes = {
		0x38, 0x07, 0x0b, 0x04, 
		0x2f, 0x10, 0x20, 0x10, 
		0x00, 0x03, 0x03, 0x00, 
		0x20, 0x30, 0x00, 0x20, 
		0x38, 0x2e, 0x15, 0x3f, 
		0x02, 0x03, 0x01, 0x02, 
		0x30, 0x00, 0x20, 0x30, 
		0x22, 0x11, 0x3f, 0x2a, 
		0x03, 0x00, 0x02, 0x03, 
		0x00, 0x30, 0x20, 0x10, 
		0x15, 0x3f, 0x01, 0x00, 
		0x00, 0x03, 0x00, 0x00, 
		0x30, 0x30, 0x30, 0x00, 
		0x3f, 0x2a, 0x2a, 0x15, 
		0x03, 0x02, 0x02, 0x01, 
    };
    constexpr byte LogoLength = 5;
    constexpr byte LogoWidth = 4 * LogoLength;
    constexpr byte LogoX = (VramWidth - LogoWidth) / 2;

    ClearScreen(); 
    HideAllSprites();
    PrintStatus();
    {
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 5 + LogoX;
        p = TitleBytes;
        repeat (LogoLength) {
            repeat (3) {
                repeat (4) {
                    *pVram = *p | 0xc0;
                    ++pVram;
                    ++p;
                }
                pVram += VramRowSize - 4;
            }
            pVram += 4 - VramRowSize * 3;
        }
    }
    PrintS(Vram + VramRowSize * 13 + 25, "PUSH SPACE KEY");
    PrintS(Vram + VramRowSize * 14 + 25, "OR ENTER KEY");
    PrintS(Vram + VramRowSize * 15 + 52, "INUFUTO 2022");
}